﻿package 
{
	import flash.display.MovieClip;
	import flash.geom.Point;
	
	
	/**
	 * 	ボールクラス
	 */
	public class Ball extends MovieClip {
		
		var moveSpd:Number;	//移動速度
		
		
		var vx:Number, vy:Number;		//移動ベクトル
		var lastX:Number, lastY:Number;	//前回座標
		
		
		/**
		 * コンストラクタ
		 * @param	spd		移動速度
		 */
		public function Ball(spd) {
			moveSpd = spd;			
		}
		
		/**
		 * 移動ベクトルを設定
		 * @param	rot		角度
		 */
		public function setVector(rot:Number):void
		{
			vx = Math.cos(rot * Math.PI / 180) * moveSpd;
			vy = Math.sin(rot * Math.PI / 180) * moveSpd;
		}
		
		/**
		 * 移動
		 */
		public function move():void
		{
			lastX = this.x;
			lastY = this.y;
			this.x += vx;
			this.y += vy;
			
		}
		
		/**
		 * ボールが場外にいるか？
		 * @return	true=ボールが場外
		 */
		public function lsOutStage():Boolean
		{
			if (this.x <= 0 || this.y <= 0 || this.x >= stage.stageWidth || this.y >= stage.stageHeight)
			{
				return true;
			}
			return false;
		}
		
		/**
		 * バーに当たった時の処理
		 * @param	bar　バーMCへの参照
		 */
		public function onHitBar(bar:MovieClip):void
		{
			this.x = lastX;
			this.y = lastY;
			
			
			var weight:Number = Math.atan2(this.y - bar.y, this.x - bar.x);
			
			if (weight / Math.PI * 180 <= -150) {
				trace("バーの左にヒット");
				weight = -230 * Math.PI / 180;
				this.x = bar.x - bar.width / 2 - this.width / 2;
			}
			else if (weight / Math.PI * 180 >= -30) {
				trace("バーの右にヒット");
				weight = 50 * Math.PI / 180;			
				this.x = bar.x + bar.width / 2 + this.width / 2;
			}
			else {
				this.y = bar.y - bar.height / 2 - this.height / 2;
			}
			vx = Math.cos(weight) * moveSpd;
			vy = Math.sin(weight) * moveSpd;
			
			move();
		}
		
		
		/**
		 * なんかに当たった時の処理
		 * @param	hitObj　当たったもの(ブロックとか壁とか)
		 */
		public function onHitObject(hitObj:MovieClip):void
		{
			this.x = lastX;
			this.y = lastY;
			
			//反転角度を判定
			var lsXBigger:Boolean = this.x >= hitObj.x;
			var lsYBigger:Boolean = this.y >= hitObj.y;
			var lsXBigger2:Boolean;
			var lsYBigger2:Boolean;
			
			//ボールが右下
			if (lsXBigger && lsYBigger) {
				//ボールの左上の点と当たった物の右下の点を判定
				lsXBigger2 = (this.x - this.width / 2)  > (hitObj.x + hitObj.width / 2);
				lsYBigger2 = (this.y - this.height / 2) > (hitObj.y + hitObj.height / 2);
				
				// 右の壁に当たっている
				if (lsXBigger2 && !lsYBigger2) {
					vx *= -1;
					this.x = hitObj.x + hitObj.width / 2 + this.width / 2;
				} 
				//下の壁に当たっている
				else if (!lsXBigger2 && lsYBigger2) {
					vy *= -1;
					this.y = hitObj.y + hitObj.height / 2 + this.height / 2;
				}
				else {
					vx *= -1;
					vy *= -1;
				}
			}
			//ボールが左下
			else if (!lsXBigger && lsYBigger)
			{
				//ボールの右上と当たった物の左下の点を判定
				lsXBigger2 = (this.x + this.width / 2)  > (hitObj.x - hitObj.width / 2);
				lsYBigger2 = (this.y - this.height / 2) > (hitObj.y + hitObj.height / 2);
				
				//左の壁に当たっている
				if (!lsXBigger2 && !lsYBigger2)	{
					vx *= -1;
					this.x = hitObj.x - hitObj.width / 2 - this.width / 2;
				}
				//下の壁に当たっている			
				else if (lsXBigger2 && lsYBigger2) {
					vy *= -1;
					this.y = hitObj.y + hitObj.height / 2 + this.height / 2;
				}
				else {
					vx *= -1;
					vy *= -1;
				}	
			}
			//ボールが右上
			else if (lsXBigger && !lsYBigger) {
				//ボールの左下と当たった物の右上の点を判定
				lsXBigger2 = (this.x - this.width / 2)  > (hitObj.x + hitObj.width / 2);
				lsYBigger2 = (this.y + this.height / 2) > (hitObj.y - hitObj.height / 2);
				
				//右の壁に当たっている
				if (lsXBigger2 && lsYBigger2) {
					vx *= -1;
					this.x = hitObj.x + hitObj.width / 2 + this.width / 2;
				}
				
				//上の壁に当たっている
				else if (!lsXBigger2 && !lsYBigger2) {
					vy *= -1;
					this.y = hitObj.y - hitObj.height / 2 - this.height / 2;
				}
				else {
					vx *= -1;
					vy *= -1;
				}
				
			}
			
			//ボールが左上
			else if (!lsXBigger && !lsYBigger) {
				//ボールの右下と当たった物の左上の点を判定
				lsXBigger2 = (this.x + this.width / 2)  > (hitObj.x - hitObj.width / 2);
				lsYBigger2 = (this.y + this.height / 2) > (hitObj.y - hitObj.height / 2);
				
				//左の壁に当たっている
				if (!lsXBigger2 && lsYBigger2) {
					vx *= -1;
					this.x = hitObj.x - hitObj.width / 2 - this.width / 2;
				}
				//上の壁に当たっている
				else if (lsXBigger2 && !lsYBigger2) {
					vy *= -1;
					this.y = hitObj.y - hitObj.height / 2 - this.height / 2;
				}
				else {
					vx *= -1;
					vy *= -1;
				}
			}
			
				
			else {
				trace("エラー：ボール反射条件外");
			}
			move();
		}
	}
	
}