﻿package {
	import flash.display.MovieClip;
	import flash.events.Event;
	import flash.events.MouseEvent;
	
	/**
	 * ブロック崩し管理クラス
	 */
	public class BreakBlock {
		
		var root:MovieClip;		
		var ball:Ball;			//ボールクラス
		
		var lsBallHold:Boolean; //ボールをキャッチした状態か？ true=キャッチ中
		
		var bar:Bar;			//バークラス
		var frameList:Array;	//枠MCの配列
		
		var blockList:Array;	//ブロックMC配列
		var breakCnt:int;		//壊したブロックの数
		
		/**
		 * コンストラクタ
		 * @param	root		ステージの参照
		 * @param	frameList	枠の参照を格納した配列
		 */
		public function BreakBlock(root:MovieClip, frameList:Array) {
			
			this.root = root;
			
			//ボール設置
			ball = new Ball(16);
			lsBallHold = true;
			root.stage.addChild(ball);
			

			//バー設置
			bar = new Bar();
			bar.y = root.stage.stageHeight - 50;
			root.stage.addChild(bar);
			
			//ブロック設置
			blockList = new Array();
			var i:uint, j:uint;
			
			var eX:Number = 65;		//ブロックを設置するの左上座標
			var eY:Number = 35;
			var xNum:uint = 5;		//ブロックの数
			var yNum:uint = 10;
			
			for (j=0; j<yNum; j++) {
				for (i=0; i<xNum; i++) {
					var block:Block = new Block();
					block.x = eX + block.width * i;
					block.y = eY + block.height* j;
					
					root.stage.addChild(block);
					blockList.push(block);
				}
			}
			breakCnt = xNum * yNum;
			
			
			this.frameList = frameList;
			
			root.stage.addEventListener(MouseEvent.MOUSE_DOWN, shotBall);
			root.stage.addEventListener(Event.ENTER_FRAME, frameFunc);			
		}
		
		
		//onEnterFrame
		public function frameFunc(event:Event):void
		{
			
			moveBar();
			
			
			if (lsBallHold == false) {
				ball.move();
				
				lsHitBlock();
				lsHitBar();
				lsHitFrame();
				
				//ボールが場外
				if (ball.lsOutStage()) {
					lsBallHold = true;
					ball.x = bar.x;
					ball.y = bar.y - bar.height / 2 - ball.height / 2;
				}
				
				//ブロックを全て壊した
				if (breakCnt == 0) {
					trace("ゲーム終了");
					remove();
					root.gotoAndPlay("_clear");
				}
				
			}
			
			//ボール発射待ち
			else {
				ball.x = bar.x;
				ball.y = bar.y - bar.height / 2 - ball.height / 2;
			}
		}
		
		//バーを移動
		function moveBar()
		{
			bar.x += bar.mouseX;
			
			//画面外にはみ出ないようにする
			if (bar.x - bar.width / 2 <= 10) {
				bar.x = bar.width / 2 + 10;
			}
			else if (bar.x + bar.width / 2 >= root.stage.stageWidth - 10) {
				bar.x = root.stage.stageWidth - bar.width / 2 - 10;
			}
		}
		
		//ボールを撃つ
		function shotBall(event:MouseEvent)
		{
			if (lsBallHold == true) {
				trace("ショット");
				lsBallHold = false;
				ball.setVector(-45);
			}
		}
	
		//ボールがバーに当たってないか
		function lsHitBar():void
		{
			if (ball.hitTestObject(bar)) {
				ball.onHitBar(bar);
			}
		}
		
		//ボールが周りの枠に当たってないか
		function lsHitFrame():void
		{
			var i:uint;
			
			for (i=0; i<frameList.length; i++) {
				if (ball.hitTestObject(frameList[i])) {
				
					ball.onHitObject(frameList[i]);
				}
			}
		}
		
		//ボールがブロックに当たってないか
		function lsHitBlock():void
		{
			var i:uint;
			
			for (i=0; i<blockList.length; i++) {
				if (blockList[i].visible == true) {
					
					//当たっている
					if (ball.hitTestObject(blockList[i])) {
						ball.onHitObject(blockList[i]);
						blockList[i].visible = false;
						
						breakCnt--;
					}
				}
			}
			
		}
		
		//ブロック。ボール。リスナを削除
		public function remove():void
		{
			var i:uint;
			
			for (i=0; i<blockList.length; i++) {
				if (blockList[i] != null) {
					root.stage.removeChild(blockList[i] as MovieClip);
					blockList[i] = null;
				}
			}
			blockList = null;
			
			root.stage.removeChild(ball);
			root.stage.removeChild(bar);
			
			root.stage.removeEventListener(Event.ENTER_FRAME, frameFunc);
			root.stage.removeEventListener(MouseEvent.MOUSE_DOWN, shotBall);
		}
		
	}
}